"""
Pyweek25 entry
Theme: Two Worlds

idea
    space arcade game
    the screen is split down the middle
    you have a ship to defend on both sides but you 
    have only one set of controls
    whatever you do on the left is also done on the right
    the enemies, on the other hand, are independent
    So as many as half of your shots and manuvers will be 
    wasted on one side.

level 1
    your two ships both face upward

level 2
    one of your ships fights attacking space ships
    the other navigates a maze
        
"""
import pygame
import random
try:
    from . import ptext
except:
    import ptext

class Sprite:
    def __init__(self, filename):
        self.image = pygame.image.load(filename)
        self.rect = self.image.get_rect()
        self.dx = 0
        self.dy = 0
        self.speed = [0, 0]
        self.max_speed = [8,8]

    def draw(self, screen):
        screen.blit(self.image, self.rect)

    def rotate(self, degrees):
        self.image = pygame.transform.rotate(self.image, degrees)

    def move(self):
        self.rect = self.rect.move(self.speed)

    def change_speed(self, dx, dy):
        # speed is the initial speed x, y
        self.dx = dx
        self.dy = dy
        newx = max(min(self.speed[0] + self.dx, self.max_speed[0]), -self.max_speed[0])
        newy = max(min(self.speed[1] + self.dy, self.max_speed[1]), -self.max_speed[1])
        self.speed = [newx, newy]

class Ship(Sprite):

    def withinx(self, maxx):
        return self.rect.x > 0 and self.rect.x < maxx - self.rect.width

    def withiny(self, maxy):
        return self.rect.y > 0 and self.rect.y < maxy - self.rect.height

class Weapon(Sprite):

    def __init__(self, filename):
        Sprite.__init__(self, filename)
        self.speed = [0, -8]
        self.live = False

class Wall():
    def __init__(self, wt, screen_size, screen):
        self.wt = wt
        self.size = screen_size
        self.x = self.size[0] // 2
        self.midline = [(self.x, 0), (self.x, self.size[1])]
        self.screen = screen
        self.black = (0,0,0)

    def update(self):
        # draw the battle field        
        pygame.draw.lines(self.screen, self.black, False, self.midline, self.wt)

class Enemy_ship(Sprite):
    def __init__(self, filename):
        Sprite.__init__(self, filename)
        self.speed = [0, 10]
        self.live = False

class Mazewall():
    def __init__(self, filename, x, y):
        self.x = x
        self.y = y
        self.image = pygame.image.load(filename)
        self.rect = self.image.get_rect()
        self.rect.centerx = x
        self.rect.centery = y

def print_opening():
    x = 100
    y = 100
    dy = 20
    message = """
    You are the only remote operator-pilot remaining. 
    You must fight the enemies from two worlds with 
    only one set of controls.
    
    Use the Arrows or WASD to pilot your ships.
    Space Bar to launch missiles.

    Press ESC once to play the game.
    Press ESC again to end.
    """

    for i, line in enumerate(message.split('\n')):
        ptext.draw(line, (x, y + i * dy), color = 'black')

def print_lev1_complete():
    x = 100
    y = 100
    dy = 20
    message = """
    Congratulations. You completed Level1.

    There is just one more level to complete.

    Press ESC once to play the game.
    Press ESC again to end.
    """

    for i, line in enumerate(message.split('\n')):
        ptext.draw(line, (x, y + i * dy), color = 'black')

def bomb(count):
    x = 100
    y = 700
    dy = 20
    message = 'Bomb Detonation in ' + str(count) + ' seconds.'

    for i, line in enumerate(message.split('\n')):
        ptext.draw(line, (x, y + i * dy), color = 'black')

def thanks():
    x = 100
    y = 300
    dy = 20
    message = 'Thanks for playing!'

    for i, line in enumerate(message.split('\n')):
        ptext.draw(line, (x, y + i * dy), color = 'black')


def wait_for_esc():
    running = True
    while running:
        # handle events
        events = pygame.event.get()
        for event in events:
            if event.type == pygame.KEYDOWN:
                # when ESC is pressed end game
                if event.type == pygame.QUIT:
                    running = False    
                elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

def main(datapath):

    pygame.init()
    
    # setup timing
    clock = pygame.time.Clock()
    FRAMES_PER_SECOND = 30
    deltat = clock.tick(FRAMES_PER_SECOND)

    screen_size = (1028, 768)

    # color definitions
    black = (0,0,0)
    white = (255, 255, 255)
    red = (255,0,0)
    green = (0,255,0)
    blue = (0,0,255)
    darkBlue = (0,0,128)
    pink = (255,200,200)
    grey = (200, 200, 200)

    screen = pygame.display.set_mode(screen_size)

    pygame.mixer.music.load(datapath + 'bullet02.ogg')

    # set up dividing wall
    wall = Wall(4, screen_size, screen)
    wall.update()

    # get 2 ships
    ships = [0, 1]
    ships[0] = Ship(datapath + 'ship01.png')
    ships[0].rect.y = int(screen_size[1] * 0.9)
    ships[1] = Ship(datapath + 'ship02.png')
    ships[1].rect.x = wall.x
    ships[1].rect.y = int(screen_size[1] * 0.9)

    # get 10 missiles
    missiles = [Weapon(datapath + 'bullet01.png') for i in range(10)]
    for missile in missiles:
        missile.dy = -8

    # get 10 enemies
    enemys = [Enemy_ship(datapath + 'enemy01.png') for i in range(10)]
    for enemy in enemys:
        enemy.dy = 8

    win = False
    lose = False
    running = True
    good_hits = 0
    enemy_time = 4000
    screen.fill(grey)
    print_opening()
    pygame.display.flip()
    wait_for_esc()
    enemy_start_timer = pygame.time.get_ticks() 
    #------------------
    while running:
        # timing
        clock.tick(FRAMES_PER_SECOND)
        enemy_new_timer = pygame.time.get_ticks()
        if enemy_new_timer - enemy_start_timer > enemy_time:
            enemy_time -= 150
            enemy_time = max(enemy_time, 500)
            enemy_start_timer = enemy_new_timer
            # launch a new enemy
            # which side to launch on?
            randx = random.randint(0,1)
            # launch a couple of enemys
            for i, enemy in enumerate(enemys):
                even = (i / 2) * 2 == i
                if not enemy.live:
                    even = (i // 2) * 2 == i
                    if even and not randx:
                        enemy.rect.x = ships[0].rect.x
                        break
                    elif randx and not even:
                        enemy.rect.x = ships[1].rect.x
                        break
            enemy.rect.y = 1
            enemy.live = True
        # handle events
        events = pygame.event.get()
        for event in events:
            if event.type == pygame.KEYDOWN:
                # when ESC is pressed end game
                if event.type == pygame.QUIT:
                    running = False    
                elif event.key == pygame.K_ESCAPE:
                    running = False
                elif event.key == pygame.K_LEFT or event.key == pygame.K_a:
                    for ship in ships:
                        ship.change_speed(-1, 0)
                elif event.key == pygame.K_RIGHT or event.key == pygame.K_d:
                    for ship in ships:
                        ship.change_speed(1, 0)
                elif event.key == pygame.K_UP or event.key == pygame.K_w:
                    for ship in ships:
                        ship.change_speed(0, -1)
                elif event.key == pygame.K_DOWN or event.key == pygame.K_s:
                    for ship in ships:
                        ship.change_speed(0, 1)
                elif  event.key == pygame.K_e:
                    # launch a couple of enemys
                    for i, enemy in enumerate(enemys[:2]):
                        even = (i / 2) * 2 == i
                        if not enemy.live:
                            even = (i // 2) * 2 == i
                            if even:
                                enemy.rect.x = ships[0].rect.x
                            else:
                                enemy.rect.x = ships[1].rect.x
                            enemy.rect.y = 1
                            enemy.live = True
                elif  event.key == pygame.K_SPACE:
                    # shoot 
                    for i in range(0, 10, 2):
                        if not missiles[i].live:
                            missiles[i].rect.center = ships[0].rect.center
                            missiles[i].live = True
                            pygame.mixer.music.play(1)

                            break
                    for i in range(1, 10, 2):
                        if not missiles[i].live:
                            missiles[i].rect.center = ships[1].rect.center
                            missiles[i].live = True
                            pygame.mixer.music.play(1)
                            break
                    # all the missiles go up

        # move things
        for ship in ships:
            ship.move()
        for missile in missiles:
            if missile.live:
                missile.move()
        for i, enemy in enumerate(enemys):
            if enemy.live:
                enemy.move()

        # check for collisions
        if running:
            for ship in ships:
                for i, enemy in enumerate(enemys):
                    if ship.rect.colliderect(enemy.rect):
                        enemy.live = False
                        lose = True
                        running = False
                        break

        if running:
            for missile in missiles:
                if missile.live:
                    for i, enemy in enumerate(enemys):
                        if enemy.live:
                            if missile.rect.colliderect(enemy.rect):
                                enemy.live = False
                                good_hits += 1
                                break
        if good_hits > 9:
            win = True
            running = False

        # ships bounce off walls
        for ship in ships:
            if ship.rect.top < 0 or ship.rect.bottom > screen_size[1]:
                ship.speed[1] = -ship.speed[1]

        if ships[0].rect.left < 0 or ships[0].rect.right > wall.x:
            ships[0].speed[0] = -ships[0].speed[0]
        if ships[1].rect.left < wall.x or ships[1].rect.right > screen_size[0]:
            ships[1].speed[0] = -ships[1].speed[0]

        # missiles disappear when they hit walls
        for missile in missiles:
            if missile.live:
                # move out of the way
                if missile.rect.bottom < 0:
                    missile.rect.center = (-100, -100)
                    missile.live = False

        # enemys disappear when they hit walls
        for enemy in enemys:
            if enemy.live:
                # move out of the way
                if enemy.rect.top > screen_size[1]:
                    enemy.rect.center = (-200, -200)
                    enemy.live = False

        # update the screen
        screen.fill(grey)
        wall.update()
        for ship in ships:
            screen.blit(ship.image, ship.rect)
        for missile in missiles:
            if missile.live:
                screen.blit(missile.image, missile.rect)
        for enemy in enemys:
            if enemy.live:
                screen.blit(enemy.image, enemy.rect)
        pygame.display.update()
        # pygame.display.flip()

    if win:
        # display win screen

        print_lev1_complete()
        pygame.display.update()
        wait_for_esc()
    elif lose: 
        # display lose screen
        winimg = pygame.image.load(datapath + 'lose.png')
        winimg_rect = winimg.get_rect()
        winimg_rect.center = (500, 350)
        screen.blit(winimg, winimg_rect)
        pygame.display.update()
        pygame.time.wait(2000)
        screen.fill(grey)
        thanks()
        pygame.display.update()
        pygame.time.wait(4000)

        return
    else: 
        # just end
        return

    # running = False

    #------------------
    # level 2

    # start bomb
    bomb_start = pygame.time.get_ticks()

    # build a maze from wall parts
    wps = []
    l1 = [(i, 100) for i in range(21, 300, 42)]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))
    l1 = [(i, 200) for i in range(screen_size[0]//2, 300, -42)]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))
    l1 = [(i, 350) for i in range(21, 400, 42)]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))

    l1 = [(i, 350+42) for i in range(21, 400, 42)][-1:]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))
    l1 = [(i, 500) for i in range(screen_size[0]//2, 100, -42)]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))
    l1 = [(i, 500-42) for i in range(screen_size[0]//2, 100, -42)][-1:]
    print(l1)
    for l in l1:
        wps.append(Mazewall(datapath + 'mazewall02.png', l[0], l[1]))

    running = True
    win = False
    for ship in ships:
        ship.rect.y = int(screen_size[1] * 0.9)

    while running:
        # timing
        clock.tick(FRAMES_PER_SECOND)
        enemy_new_timer = pygame.time.get_ticks()
        if enemy_new_timer - enemy_start_timer > enemy_time:
            enemy_time -= 150
            enemy_time = max(enemy_time, 500)
            enemy_start_timer = enemy_new_timer
            # launch a new enemy
            # which side to launch on?
            randx = 1
            # launch a couple of enemys
            for i, enemy in enumerate(enemys):
                even = (i / 2) * 2 == i
                if not enemy.live:
                    even = (i // 2) * 2 == i
                    if even and not randx:
                        enemy.rect.x = ships[0].rect.x
                        break
                    elif randx and not even:
                        enemy.rect.x = ships[1].rect.x
                        break
            enemy.rect.y = 1
            enemy.live = True
        # handle events
        events = pygame.event.get()
        for event in events:
            if event.type == pygame.KEYDOWN:
                # when ESC is pressed end game
                if event.type == pygame.QUIT:
                    running = False    
                elif event.key == pygame.K_ESCAPE:
                    running = False
                elif event.key == pygame.K_LEFT or event.key == pygame.K_a:
                    for ship in ships:
                        ship.change_speed(-1, 0)
                elif event.key == pygame.K_RIGHT or event.key == pygame.K_d:
                    for ship in ships:
                        ship.change_speed(1, 0)
                elif event.key == pygame.K_UP or event.key == pygame.K_w:
                    for ship in ships:
                        ship.change_speed(0, -1)
                elif event.key == pygame.K_DOWN or event.key == pygame.K_s:
                    for ship in ships:
                        ship.change_speed(0, 1)
                elif  event.key == pygame.K_e:
                    # launch a couple of enemys
                    for i, enemy in enumerate(enemys[:2]):
                        even = (i / 2) * 2 == i
                        if not enemy.live:
                            even = (i // 2) * 2 == i
                            if even:
                                enemy.rect.x = ships[0].rect.x
                            else:
                                enemy.rect.x = ships[1].rect.x
                            enemy.rect.y = 1
                            enemy.live = True
                elif  event.key == pygame.K_SPACE:
                    # shoot 
                    for i in range(0, 10, 2):
                        if not missiles[i].live:
                            missiles[i].rect.center = ships[0].rect.center
                            missiles[i].live = True
                            break
                    for i in range(1, 10, 2):
                        if not missiles[i].live:
                            missiles[i].rect.center = ships[1].rect.center
                            missiles[i].live = True
                            break
                    # all the missiles go up

        # move things
        for ship in ships:
            ship.move()
        for missile in missiles:
            if missile.live:
                missile.move()
        for i, enemy in enumerate(enemys):
            if enemy.live:
                enemy.move()

        # check for collisions
        if running:
            # ships hitting enemies
            for ship in ships:
                for i, enemy in enumerate(enemys):
                    if ship.rect.colliderect(enemy.rect):
                        enemy.live = False
                        lose = True
                        running = False
                        break

        if running:
            # missiles hitting enemies
            for missile in missiles:
                if missile.live:
                    for i, enemy in enumerate(enemys):
                        if enemy.live:
                            if missile.rect.colliderect(enemy.rect):
                                enemy.live = False
                                good_hits += 1
                                break

        # ships bounce off walls
        for ship in ships:
            # top or bottom
            if ship.rect.top < 0 or ship.rect.bottom > screen_size[1]:
                ship.speed[1] = -ship.speed[1]

        # side walls
        if ships[0].rect.left < 0 or ships[0].rect.right > wall.x:
            ships[0].speed[0] = -ships[0].speed[0]
        if ships[1].rect.left < wall.x or ships[1].rect.right > screen_size[0]:
            ships[1].speed[0] = -ships[1].speed[0]

        # maze walls
        for wp in wps:
            if ships[0].rect.colliderect(wp.rect):
                # which side of the ship hit the maze?
                # right
                if wp.rect.collidepoint(ships[0].rect.midright):
                    # bounce to the side
                    ships[0].speed[0] = -ships[0].speed[0]
                # left
                elif wp.rect.collidepoint(ships[0].rect.midleft):
                    # bounce to the side
                    ships[0].speed[0] = -ships[0].speed[0]
                # top
                if wp.rect.collidepoint(ships[0].rect.midtop):
                    # bounce to the side
                    ships[0].speed[1] = -ships[0].speed[1]
                # bottom
                elif wp.rect.collidepoint(ships[0].rect.midbottom):
                    # bounce to the side
                    ships[0].speed[1] = -ships[0].speed[1]

        # missiles disappear when they hit walls
        for missile in missiles:
            if missile.live:
                # move out of the way
                if missile.rect.bottom < 0:
                    missile.rect.center = (-100, -100)
                    missile.live = False

        # enemys disappear when they hit walls
        for enemy in enemys:
            if enemy.live:
                # move out of the way
                if enemy.rect.top > screen_size[1]:
                    enemy.rect.center = (-200, -200)
                    enemy.live = False


        # update the screen
        screen.fill(grey)

        # update bomb
        bomb_next = pygame.time.get_ticks()
        bomb_elapsed = 100 - ((bomb_next - bomb_start) // 1000)
        bomb(bomb_elapsed)

        print(ships[0].rect.y)
        if ships[0].rect.y < 10:
            win = True
            running = False
        elif bomb_elapsed < 1:
            lose = True
            running = False


        # maze
        for wp in wps:
            screen.blit(wp.image, wp.rect)
        # dividing wall
        wall.update()
        for ship in ships:
            screen.blit(ship.image, ship.rect)
        for missile in missiles:
            if missile.live:
                screen.blit(missile.image, missile.rect)
        for enemy in enemys:
            if enemy.live:
                screen.blit(enemy.image, enemy.rect)
        pygame.display.update()
        # pygame.display.flip()

    if win:
        # display win screen
        winimg = pygame.image.load(datapath + 'win.png')
        winimg_rect = winimg.get_rect()
        winimg_rect.center = (500, 350)
        screen.blit(winimg, winimg_rect)
        pygame.display.update()
        wait_for_esc()
    elif lose: 
        # display lose screen
        winimg = pygame.image.load(datapath + 'lose.png')
        winimg_rect = winimg.get_rect()
        winimg_rect.center = (500, 350)
        screen.blit(winimg, winimg_rect)
        pygame.display.update()
        pygame.time.wait(2000)
        screen.fill(grey)
        thanks()
        pygame.display.update()
        pygame.time.wait(4000)

    else: 
        # just end
        pass

if __name__ == "__main__":
    # this path is for local testing
    datapath = '../data/'
    main(datapath)
